function [Vectors,Values,Psi] = azimuth(signalname)
% =================================================
% azimuth: bss based on azimuth discrimination and synthesis.
% reference: "Sound Souce Separation: azimuth discrimination 
% and resynthesis". D. Barry, B. Lawlor, and E. Coyle: Dafx04.
% author: Mijail Guillemard

% --> set path variables 
addpath('./');

% --> load audio test files
% sectionSignal = 1:90000;
sectionSignal = 1:90000;
% sectionSignal = sectionSignal + 150000;
sectionSignal = sectionSignal + 150000;
% sectionSignal = sectionSignal + 40000;
[signal1 sr] = wavread('audio/odoors.wav'); % signal1 = sigxnal1(sectionSignal);
sch1 = signal1(sectionSignal,1); % signal channel 1 
sch2 = signal1(sectionSignal,2); % signal channel 2
% --> files saving administation
tagfile = '_azm_t9d_'; fname = ['hz' tagfile]; sourcefilename = 'azimuth.m'; 
pathfile = './results/';
filename = [pathfile fname]; % copy the current source file
% copyfile(sourcefilename,[pathfile fname sourcefilename]);

% --> prepare stereo signal with known gain coefficients
% [sch1, sch2, sr] = prepareStereoSignal(sectionSignal);

schL = sch1 - sch2; % mid side - left right 
schR = sch1 + sch2; 

% --> general comment: (a copy of this file be saved with rendered audio)
% preparing azimuth discrimination code

% azimuth algorithm
% =================

% --> build two spectrograms: one per channel
fftSize = 1024*4; winSize = 1024*4; hopSize = 256*4; % xxparameters
ssch1 = stft(sch1',fftSize,winSize,hopSize);
ssch2 = stft(sch2',fftSize,winSize,hopSize);
assch1 = abs(ssch1);
assch2 = abs(ssch2);

% --> build azimuth-frequency function: azr
azres = 150; % azimuth resolution: xxparameters
[ln,lm] = size(assch1); % ln: frequency bins, lm: time frames
% gs = linspace(1,azres,azres)/azres; % gain scaling values % for 2partials and Mix1
% gs = linspace(1,azres,azres)/azres*1; % gain scaling values % for 2partials and Mix1
% gs = linspace(1,azres,azres)/azres*50; % gain scaling values % testing
% gs = linspace(-1,1.3,azres); % gain scaling values % testing
gs = linspace(-0.4,3,azres); % gain scaling values % testing
% gs = linspace(10,40,azres)/azres; % gain scaling values % testing
% gs = linspace(1,azres,azres)/azres*1; % gain scaling values % testing
% gs = linspace(20,60,azres)/azres*2; % gain scaling values % testing
azr = zeros(ln,azres,lm); % initialize first - might be faster
for k=1:lm azr(:,:,k) = abs(ssch1(:,k)*ones(1,azres) - ssch2(:,k)*gs); end;
% for k=1:lm azr(:,:,k) = abs(ssch2(:,k)*ones(1,azres) - ssch1(:,k)*gs); end;
% figure(6); imagesc(azr(:,:,10)); 

% matrix plot
azsum1 = sum(azr(:,:,:));
azsum1 = squeeze(azsum1);
figure(1); imagesc(azsum1); 

% --> transform azimuth-frequency function (nulls will become peaks) azr2
azr2 = zeros(ln,azres,lm);
for k=1:lm
    azr2min = min(azr(:,:,k)');
    azr2max = max(azr(:,:,k)');
    azr2diff = azr2max-azr2min;
    uk = azr2min'*ones(1,azres);
    bk = azr2diff'*ones(1,azres);
    azr2(:,:,k) = (azr(:,:,k) == uk ).*bk;
end;

% figure(6); imagesc(azr2(:,:,10)); 

% figure(1); surf(azr2(:,:,50)); grid on; rotate3d on;
% colormap(cool(2)); % colormap('default');
% figure(5); imagesc(20*log10(flipud(assch2)));

% --> prepare resynthesis stage
% plotting alternative functions: (1D versions)
azsum = sum(azr2(:,:,:));
azsum = squeeze(azsum);
figure(2); imagesc(azsum); % figure(2); image(azsum); 
ap1 = sum(azsum');
% figure(3); plot(ap1(10:azres-10));
figure(3); plot(ap1);
figure(4); plot(gs);

% --> manual resynthesis of a signal component
d = 58; H = 6; % selected azimuth range
spectR = zeros(ln,lm); 
for k=1:lm spectR(:,k) = sum(azr2(:,d-H:d+H,k)')'; end;
spectI = angle(ssch1); 
spect = spectR.*exp(i*spectI); 
sgn = istft(spect,fftSize,winSize,hopSize);
wavwrite(sgn,sr,[filename 'sgn4a.wav']); % istft regenerated sgnl 

% --> save analyzed section 
ssk = [sch1,sch2]; 
wavwrite(ssk,sr,[filename 'origSection.wav']); % istft regenerated sgnl 

keyboard;

% % --> prepare a movie (notice the usage of 'filename', and avifile constructs)
% aviobj = avifile([filename 'movie5.avi'],'fps',5); 
% for kio = 1:2:lm;
%     figure(4); surf(azr2(:,:,kio)); grid on;
%     % set(gca,'xlim',[-40,40]); 
%     % set(gca,'ylim',[-40,40]); 
%     set(gca,'zlim',[1,80]);
%     frame = getframe(gca);
%     aviobj = addframe(aviobj,frame);
% end;
% aviobj = close(aviobj);
% disp('done-1'); keyboard; 

