function [Vectors,Values,Psi] = isaSignalDetection(signalname)
% ========================================================
% signalManifold: constructs a manifold from a  mono signal 
% using pca, ica, or isomap methods. 
% author: Mijail Guillemard

close all; clear all;

% set path variables for ica and isomap
% =====================================
addpath('./');

addpath('./drtoolbox/');
addpath('./drtoolbox/techniques/');

% load audio test files
% =====================
sectionSignal = 10000:200000;
% signal3 = wavread('audio/a3.wav'); signal3 = signal3(sectionSignal);
signale = wavread('audio/h2.wav'); signale = signale(sectionSignal);
signal3a = wavread('audio/cymbal4.wav'); %signal3 = signal3(sectionSignal-sectionSignal(1)+1);
signal3a = wavread('audio/cymbal8.wav'); %signal3 = signal3(sectionSignal-sectionSignal(1)+1);
signal3a = wavread('audio/cymbal3.wav'); %signal3 = signal3(sectionSignal-sectionSignal(1)+1);
signal3 = wavread('audio/sideDrum.wav'); % signale = signale(sectionSignal);

% ==> files saving administation
% ==============================
tagfile = '-';        sourcefilename = 'signalManifold2.m'; 
fname = ['dimred'];  pathfile = './results/';
filename = [pathfile fname]; % for coping the current source file

% assemble audio files for analysis
% =================================
intvk = 1:100000;
sg1a = signal3(intvk); sg2 = signale(intvk); sg3a = signal3a(intvk); 
sg1 = sg1a + sg3a; sg1 = sg3a; 
signal = sg1 + sg2; 
% signal = signale;
% signal = signal(1:100000);
% wavwrite(sg1a,44100,[filename 'sg1.wav']);wavwrite(sg2,44100,[filename 'sg2.wav']);wavwrite(sg3a,44100,[filename 'sg3.wav']);

% stftReducedPlot(sg1a,'01'); stftReducedPlot(sg2,'02'); stftReducedPlot(sg3a,'03'); 
% stftReducedPlot3(signal,'12');
% paperPlots1(sg1a,sg2,sg3a); 

% ISA algorithm version 2
% =======================
fftSize = 1024/2; winSize = 1024/2; hopSize = 256/2; nic = 10;
% fftSize = 1024/2; winSize = 1024/2; hopSize = 256/4; nic = 20;
% fftSize = 1024/2; winSize = 1024/2; hopSize = 256/4; nic = 6;
fftSize = 1024/2; winSize = 1024/2; hopSize = 256/4;  nic = 22;
fftSize = 1024/2; winSize = 1024/2; hopSize = 256/4;  nic = 15;
fftSize = 1024/2; winSize = 1024/2; hopSize = 256/4;  nic = 10;
stftSignal = stft(signal',fftSize,winSize,hopSize);
astftA = abs(stftSignal); % astft = stftSignal;
% astft = blockFormat(astftA); astft = log10(astft);  
astft = astftA; % astft = log10(astftA); 
svrs.fftSize = fftSize; svrs.winSize = winSize; svrs.hopSize = hopSize; 
svrs.stftSignal = stftSignal; 
svrs.sg1 = sg1; svrs.sg2 = sg2; svrs.filename = filename;

% %%  check that we are correctly reconstructing the original signal 
% %%  for a correct reconstrution set fftSize == winSize, hopSize is irrelevant
sgnaa = istft(stftSignal,fftSize,winSize,hopSize); 
wavwrite(0.99*sgnaa/max(abs(sgnaa)),44100,[filename 'gen.wav']); % istft regenerated signal 

%% %% combine first and second signals in different blocks for testing dimRed methods
%% fsg1 = stft(sg1',fftSize,winSize,hopSize);
%% afsg1 = abs(fsg1); % astft = stftSignal;
%% fsg2 = stft(sg2',fftSize,winSize,hopSize);
%% afsg2 = abs(fsg2); % astft = stftSignal;
%% X = [afsg1 afsg2]; astft = X; 
%% meanPos = mean(X')'; X = X - repmat(meanPos,1,size(X,2)); 

% ISA according to Uhle, Dittmar, Sporer - Step 1: Dimensional Reduction
% ======================================================================
Xsed = astft; [a,b] = size(Xsed);
meanPos = mean(Xsed')'; Xsed = Xsed - repmat(meanPos,1,size(Xsed,2)); 
[U,D,V] = svd(Xsed',0); % testing PCA projection 
Tvec = D(1:nic,:)*V';
Xh = Tvec*Xsed; Xhpca = Xh; disp('basic PCA ready'); 
labels = linspace(0,1,size(Xh,2)); 
% figure, scatter3(Xh(1,:), Xh(2,:), Xh(3,:), 20, labels,'filled'); title('Result PCA1'), drawnow; rotate3d on;
% X = Xh; nic = 4;

%% %% check that the there are not repeated vectors in the dataset
%% dst = L2_distance(X,X);
%% dst = dst + eye(size(dst,1));
%% disp('ggft'); keyboard;

% ==> testing different dim red techniques with dimRed toolbox
% ============================================================
no_dims = nic; 
% techN = {'PCA', 'Laplacian','LLTSA','LTSA','LLE'};
techN = {'PCA','Laplacian','LLTSA'};
% techN = {'PCA','Laplacian'};
techN = {'Laplacian'};
% techN = {'PCA','LLTSA'};
techN = {'Isomap'};
%techN = {'PCA','Laplacian','Isomap'};
%techN = {'Laplacian','Isomap','PCA'};
% techN = {'LTSA'};
% techN = {'HessianLLE','LandmarkIsomap','FastMVU'}; % these methods are not working well
% techN = {'LLE'};

for kt=1:length(techN)
  tag = techN(kt); tag = tag{1}; 
  dimRedGabor(Xsed,tag,nic,Tvec,svrs);
end;

disp('dim. red. techniques tested - results saved in ./images/ '); keyboard; 

