function [Vectors,Values,Psi] = isaSignal(signalname)
% ========================================================
% signalManifold: constructs a manifold from a  mono signal 
% using pca, ica, or isomap methods. 
% author: Mijail Guillemard

% set path variables for ica and isomap
% =====================================
addpath('fastica');
addpath('isomap');
addpath('./');

% load audio test files
% =====================
sectionSignal = 10000:200000;
signal3 = wavread('audio/a3.wav'); signal3 = signal3(sectionSignal);
signale = wavread('audio/h2.wav'); signale = signale(sectionSignal);

% ==> files saving administation
tagfile = '_d1_';        sourcefilename = 'signalManifold2.m'; 
fname = ['a3h2' tagfile];  pathfile = './results/';
filename = [pathfile fname]; % for coping the current source file

% assemble audio files for analysis
% =================================
signal = signal3 + signale; 

% ISA algorithm version 2
% =======================
fftSize = 1024/2; winSize = 1024/2; hopSize = 256/2; nic = 15;
stftSignal = stft(signal',fftSize,winSize,hopSize);
astft = abs(stftSignal); % astft = stftSignal;

% ISA according to Uhle, Dittmar, Sporer
% ======================================
X = astft; [a,b] = size(X); NUM_IC = nic;
[U,D,V] = svd(X',0);
T = D(1:nic,:)*V';
Xh = T*X;
[A,XX] = jade(Xh, NUM_IC); Ainv = pinv(A); % means are not deleted 
% [icasig,Am,Ainv,whitenMatrixm] = FASTICA (Xh, 'lastEig', nic, 'numOfIC', nic); % means are deleted
E = Ainv*Xh;
Finv = Ainv*T;
tweigths = E;
fweigths = pinv(Finv);

% ==> build components by with new spectrograms using tweigth and fweigth
clear sgns sgn;
for k=1:nic; sgns(k,:,:) = fweigths(:,k)*tweigths(k,:); end;
for k=1:nic; sgns(k,:,:) = squeeze(sgns(k,:,:)).*exp(i*angle(stftSignal)); end;
for k=1:nic; sgn(k,:) = istft(squeeze(sgns(k,:,:)),fftSize,winSize,hopSize); end;

% ==> save components and rebuilded signals 
sgnaa = istft(stftSignal,fftSize,winSize,hopSize); 
wavwrite(sgnaa,44100,[filename 'gen.wav']); % istft regenerated signal 
wavwrite(signal,44100,[filename 'signal.wav']); % original singnal
ak = sum(sgn); wavwrite(ak,44100,[filename 'sgnSum.wav']); % sum of all components
fact = 1; % amplitude factor 
for k=1:nic; wavwrite(fact*sgn(k,:),44100,[filename 'sgnf' num2str(k) '.wav']); end;

% % % ==> summing manually components: a6: 4 drums 
casta = 0; drum2 = 0; 
for k=[4,9,14]; casta = casta + sgn(k,:); end; 
for k=[1,2,3,5,6,7,8,10,11,12,13]; drum2 = drum2 + sgn(k,:); end; 
wavwrite(casta,44100,[filename 'casta' '.wav']);
wavwrite(drum2,44100,[filename 'drum2' '.wav']);

% ==> plot signal and some components
% set(gcf,'PaperSize',[30,59]);
figure(1);
% set(gcf,'Position',[420 140 700 800]);
subplot(4,1,1); plot(signal); grid on;
subplot(4,1,2); plot(sgn(1,:)); grid on;
subplot(4,1,3); plot(sgn(2,:)); grid on;
subplot(4,1,4); plot(sgn(3,:)); grid on;

