function [Vectors,Values,Psi] = isaSignal(signalname)
% ========================================================
% signalManifold: constructs a manifold from a  mono signal 
% using pca, ica, or isomap methods. 
% author: Mijail Guillemard

close all; clear all;

% set path variables for ica and isomap
% =====================================
addpath('fastica');
addpath('isomap');
addpath('./');

addpath('./drtoolbox/');
addpath('./drtoolbox/techniques/');

% load audio test files
% =====================
sectionSignal = 10000:200000;
signal3 = wavread('audio/a3.wav'); signal3 = signal3(sectionSignal);
signale = wavread('audio/h2.wav'); signale = signale(sectionSignal);
signal3a = wavread('audio/cymbal4.wav'); %signal3 = signal3(sectionSignal-sectionSignal(1)+1);
signal3 = wavread('audio/sideDrum.wav'); % signale = signale(sectionSignal);

% ==> files saving administation
tagfile = '_d1_';        sourcefilename = 'signalManifold2.m'; 
fname = ['a3h2' tagfile];  pathfile = './results/';
filename = [pathfile fname]; % for coping the current source file

% assemble audio files for analysis
% =================================
intvk = 1:100000;
sg1a = signal3(intvk); sg2 = signale(intvk); sg3a = signal3a(intvk); 
sg1 = sg1a + sg3a;
signal = sg1 + sg2; 
% signal = signale;
% signal = signal(1:100000);
% wavwrite(sg1a,44100,[filename 'sg1.wav']);wavwrite(sg2,44100,[filename 'sg2.wav']);wavwrite(sg3a,44100,[filename 'sg3.wav']);

% stftReducedPlot(sg1a,'01'); stftReducedPlot(sg2,'02'); stftReducedPlot(sg3a,'03'); 
% stftReducedPlot3(signal,'12');

% ISA algorithm version 2
% =======================
fftSize = 1024/2; winSize = 1024/2; hopSize = 256/2; nic = 15;
fftSize = 1024/2; winSize = 1024/2; hopSize = 256/4; nic = 20;
fftSize = 1024/2; winSize = 1024/2; hopSize = 256/4; nic = 10;
stftSignal = stft(signal',fftSize,winSize,hopSize);
astftA = abs(stftSignal); % astft = stftSignal;
% astft = blockFormat(astftA); 
astft = astftA; 

%% %% combine first and second signals in different blocks for testing dimRed methods
%% fsg1 = stft(sg1',fftSize,winSize,hopSize);
%% afsg1 = abs(fsg1); % astft = stftSignal;
%% fsg2 = stft(sg2',fftSize,winSize,hopSize);
%% afsg2 = abs(fsg2); % astft = stftSignal;
%% X = [afsg1 afsg2]; astft = X; 
%% meanPos = mean(X')'; X = X - repmat(meanPos,1,size(X,2)); 

% ISA according to Uhle, Dittmar, Sporer - Step 1: Dimensional Reduction
% ======================================
X = astft; [a,b] = size(X);
meanPos = mean(X')'; X = X - repmat(meanPos,1,size(X,2)); 
[U,D,V] = svd(X',0);
T = D(1:nic,:)*V';
Xh = T*X; Xhpca = Xh; disp('basic PCA ready'); 
labels = linspace(0,1,size(Xh,2)); sctBs = 20; % size of the scatter3 point
figure, scatter3(Xh(1,:), Xh(2,:), Xh(3,:), sctBs, labels,'filled'); title('Result PCA1'), drawnow; rotate3d on;

%%== % ==> testing different dim red techniques 
no_dims = nic; 
%% techN = {'PCA', 'Laplacian','LLTSA'};
%% for kt=1:length(techN)
%%   tag = techN(kt); tag = tag{1}; 
%%   dimRedGabor(X,tag,nic,sctBs,T);
%% end;
%% 
%% disp('fdee55dfhryt544'); keyboard; 

% ISA according to Uhle, Dittmar, Sporer - Step 2: ICA separation
% ======================================
[A,XX] = jade(Xh, nic); Ainv = pinv(A); % means are not deleted 
% [icasig,Am,Ainv,whitenMatrixm] = FASTICA (Xh, 'lastEig', nic, 'numOfIC', nic); % means are deleted
E = Ainv*Xh;
Finv = Ainv*T;
tweigths = E; 
fweigths = pinv(Finv); % fweigths = fweigths + repmat(meanPos,1,size(fweigths,2));

% deblockFreqMatrix(fweigths',stftSignal,filename);
disp('ica step ready'); keyboard; 

% ==> build components by with new spectrograms using tweigth and fweigth
clear sgns sgn;
for k=1:nic; sgns1(k,:,:) = fweigths(:,k)*tweigths(k,:)/max(tweigths(k,:))/2; end;
% for k=1:nic; sgns1(k,:,:) = fweigths(:,k)*tweigths(k,:); end;
% for k=1:nic; sgns(k,:,:) = (squeeze(sgns1(k,:,:))+repmat(meanPos,1,size(X,2))).*exp(i*angle(stftSignal)); end;
for k=1:nic; sgns(k,:,:) = squeeze(sgns1(k,:,:)).*exp(i*angle(stftSignal)); end;
for k=1:nic; sgn(k,:) = istft(squeeze(sgns(k,:,:)),fftSize,winSize,hopSize); end;

% ==> save components and rebuilded signals 
sgnaa = istft(stftSignal,fftSize,winSize,hopSize); 
wavwrite(sgnaa,44100,[filename 'gen.wav']); % istft regenerated signal 
wavwrite(signal,44100,[filename 'signal.wav']); % original singnal
ak = sum(sgn); wavwrite(ak,44100,[filename 'sgnSum.wav']); % sum of all components
fact = 1; % amplitude factor 
for k=1:nic; wavwrite(fact*sgn(k,:),44100,[filename 'sgnf' num2str(k) '.wav']); end;

disp('ungrouped signals saved'); keyboard; 

sdnGrouped = sdrGrouping(sgn,sg1,sg2); keyboard; 

% % % ==> summing manually components: a6: 4 drums 
casta = 0; drum2 = 0; 
for k=[4,9,14]; casta = casta + sgn(k,:); end; 
for k=[1,2,3,5,6,7,8,10,11,12,13]; drum2 = drum2 + sgn(k,:); end; 
wavwrite(casta,44100,[filename 'casta' '.wav']);
wavwrite(drum2,44100,[filename 'drum2' '.wav']);

% ==> plot signal and some components
% set(gcf,'PaperSize',[30,59]);
figure(1);
% set(gcf,'Position',[420 140 700 800]);
subplot(4,1,1); plot(signal); grid on;
subplot(4,1,2); plot(sgn(1,:)); grid on;
subplot(4,1,3); plot(sgn(2,:)); grid on;
subplot(4,1,4); plot(sgn(3,:)); grid on;
%subplot(8,1,5); plot(sgn(4,:)); grid on;
%subplot(8,1,6); plot(sgn(5,:)); grid on;
%subplot(8,1,7); plot(sgn(6,:)); grid on;
%subplot(8,1,8); plot(sgn(7,:)); grid on;
% print -depsc basisSgn;
% fn = [filename 'basisSgn']; print(gcf,'-dbitmap', fn);

% figure(2); plot(newVectorsProj(1,:),newVectorsProj(2,:),'r.'); grid on;
% print -depsc cloudPca;
% print -dbitmap [filename cloudPca];
% fn = [filename 'cloudPca']; print(gcf,'-dbitmap', fn);

disp('done3'); keyboard;

