function [Vectors,Values,Psi] = signalManifold(signalname)
% ========================================================
% signalManifold: constructs a manifold from a  mono signal 
% using pca, ica, or isomap methods. 
% author: Mijail Guillemard
% June 2005

% set path variables for ica and isomap
% =====================================
%% path(path,'d:\mguillem\job\bss\mfiles\');
%% path(path,'d:\mguillem\job\bss\mfiles\fastica\');
%% path(path,'d:\mguillem\job\bss\mfiles\isomap\');
addpath('./');

% load audio test files
% =====================
% signal1 = wavread(signalname);
% sectionSignal = 10000:210000;
sectionSignal = 10000:200000;
signal1 = wavread('audio/a1.wav'); signal1 = signal1(sectionSignal);
signal2 = wavread('audio/a2.wav'); signal2 = signal2(sectionSignal);
signal3 = wavread('audio/a3.wav'); signal3 = signal3(sectionSignal);
signal4 = wavread('audio/a4.wav'); signal4 = signal4(sectionSignal);
signal5 = wavread('audio/a5.wav'); signal5 = signal5(sectionSignal);
signal6 = wavread('audio/a6.wav'); signal6 = signal6(sectionSignal);
signal7 = wavread('audio/a7.wav'); signal7 = signal7(sectionSignal);
signal8 = wavread('audio/a8.wav'); signal8 = signal8(sectionSignal);

signalc = wavread('audio/h3.wav'); signalc = signalc(sectionSignal);
% signald = wavread('audio/g.wav'); signald = signald(sectionSignal);
signalh2 = wavread('audio/h2.wav'); signalh2 = signalh2(sectionSignal);

% assemble audio files for analysis
% =================================
% signal = signalh2 + signal5; 
signal = signalh2 + signal3; 
% signal = signal5; 
% signal = signal(76764:90816); 
% signal = signal(30764:99816); 
% signal = signal(30764:41816); 
signal = signal(30764:99816); 

filename = 'a6k5_';

% ISA algorithm version 2
% =======================
% fftSize = 2*1024; winSize = 2*1024; hopSize = 512; nic = 8;
fftSize = 1024; winSize = 1024; hopSize = 512/2; nic = 20;
% fftSize = 4*128; winSize = 4*128; hopSize = 4*32; nic = 15;
stftSignal = stft(signal',fftSize,winSize,hopSize);
astft = abs(stftSignal); % astft = stftSignal;

% ISA algorithm version 3: casey mpeg7 code
% =========================================
X = astft'; [a,b] = size(X); NUM_IC = nic;
% X = 10*log10(X+realmin);
% env =  sqrt(sum(X'.*X')'); % 2-norm of log magnitude spectrum
% X = X ./ (env*ones(1,b));
[U,A,V] = svd(X,0);
V=V(:,1:NUM_IC);
% V = pc_evectors(X',NUM_IC);
Y=X*V; 
[V2,XX] = jade(Y', NUM_IC);
V=V*pinv(V2)';
% tweigths = Y'; fweigths = V; % a6z1
tweigths = XX; fweigths = V; % a6z2
%clear sgns sgn;

% ==> build components signals by weighting original spectrogram
clear sgns sgn;
tws = abs(tweigths); % prepare scale weights
for k=1:nic; tws(k,:) = abs(tws(k,:))/max(tws(k,:)); end; % prepare scale weights
for k=1:nic; sgns(k,:,:) = stftSignal.*repmat(tws(k,:),size(stftSignal,1),1); end;
for k=1:nic; sgn(k,:) = istft(squeeze(sgns(k,:,:)),fftSize,winSize,hopSize); end;

% ==> resinthesis signals 
sgnaa = istft(stftSignal,fftSize,winSize,hopSize); wavwrite(sgnaa,44100,[filename 'gen.wav']);
wavwrite(signal,44100,[filename 'signal.wav']);
ak = sum(sgn); wavwrite(ak,44100,[filename 'sgnSum.wav']);
fact = 0.5;
for k=1:nic; wavwrite(fact*sgn(k,:),44100,[filename 'sgnf' num2str(k) '.wav']); end;

% set(gcf,'PaperSize',[30,59]);
figure(1);
% set(gcf,'Position',[420 140 700 800]);
subplot(4,1,1); plot(signal); grid on;
subplot(4,1,2); plot(sgn(1,:)); grid on;
subplot(4,1,3); plot(sgn(2,:)); grid on;
subplot(4,1,4); plot(sgn(3,:)); grid on;
%subplot(8,1,5); plot(sgn(4,:)); grid on;
%subplot(8,1,6); plot(sgn(5,:)); grid on;
%subplot(8,1,7); plot(sgn(6,:)); grid on;
%subplot(8,1,8); plot(sgn(7,:)); grid on;
% print -depsc basisSgn;
fn = [filename 'basisSgn']; print(gcf,'-dbitmap', fn);

% figure(2); plot(newVectorsProj(1,:),newVectorsProj(2,:),'r.'); grid on;
% print -depsc cloudPca;
% print -dbitmap [filename cloudPca];
fn = [filename 'cloudPca']; print(gcf,'-dbitmap', fn);

% ==> summing manually components: h23 (castanets + scales) 
scales = 0; casta = 0; 
for k=[1,3,4,5,6,10,11]; scales = scales + sgn(k,:); end; 
for k=[2,7,8,9,12,13,14,15]; casta = casta + sgn(k,:); end; 
wavwrite(scales,44100,[filename 'scalesSum' '.wav']);
wavwrite(casta,44100,[filename 'castaSum' '.wav']);
scales = 0; casta = 0; 
for k=[1,3,4,5,6,8,10,11]; scales = scales + sgn(k,:); end; 
for k=[2,7,9,11,12,13,14,15,17]; casta = casta + sgn(k,:); end; 
wavwrite(scales,44100,[filename 'scalesSum' '.wav']);
wavwrite(casta,44100,[filename 'castaSum' '.wav']);

% % ==> summing manually components: a6: 4 drums 
drum1 = 0; drum2 = 0; drum3 = 0; drum4 = 0;
% undecided: 10
for k=[1,3,7]; drum1 = drum1 + sgn(k,:); end; 
for k=[4,5,9,11,12,14,15]; drum2 = drum2 + sgn(k,:); end; 
for k=[2,8,13]; drum3 = drum3 + sgn(k,:); end; 
for k=[6]; drum4 = drum4 + sgn(k,:); end; 
wavwrite(drum1,44100,[filename 'drum1' '.wav']);
wavwrite(drum2,44100,[filename 'drum2' '.wav']);
wavwrite(drum3,44100,[filename 'drum3' '.wav']);
wavwrite(drum4,44100,[filename 'drum4' '.wav']);


disp('printing3'); keyboard;

